use hyper::StatusCode;
use std::sync::Arc;
use tracing::{instrument, log::*};

use axum::{
    response::{Html, IntoResponse, Response},
    Extension,
};

use crate::{State, WebsiteError};

#[instrument(skip(state))]
pub async fn index(Extension(state): Extension<Arc<State>>) -> std::result::Result<Html<Vec<u8>>, WebsiteError> {
    let ctx = tera::Context::new();
    let res = state.tera.render("index.html", &ctx).map_err(|e| {
        error!("Failed rendering index: {}", e);
        WebsiteError::NotFound
    })?;
    Ok(Html(res.into()))
}

impl IntoResponse for WebsiteError {
    fn into_response(self) -> Response {
        match self {
            WebsiteError::NotFound => {
                info!("not found");
                (StatusCode::NOT_FOUND, ()).into_response()
            }
            WebsiteError::InternalError(e) => {
                error!("internal error: {e}");
                (StatusCode::INTERNAL_SERVER_ERROR, ()).into_response()
            }
        }
    }
}
